﻿var TodoList = TodoList || {};

TodoList.Priority = {
    VeryLow: 1,
    Low: 2,
    Normal: 3,
    High: 4,
    VeryHigh: 5
};

// Define TodoListSettings: the global object for application settings
var TodoListSettings = WinJS.Class.define(function () {
    var localSettings = Windows.Storage.ApplicationData.current.localSettings;
    var that = {};
    that.defaultPriority = TodoList.Priority.Normal;
    that.load = function () {
        _loadFromSettings();
    };
    that.save = function () {
        _saveToSettings();
    };

    function _loadFromSettings() {
        var priority = localSettings.values["defaultPriority"];
        if (priority) {
            that.defaultPriority = priority;
        }
    };

    function _saveToSettings() {
        localSettings.values["defaultPriority"] = that.defaultPriority;
    };

    return that;
});
TodoList.settings = new TodoListSettings();

// Define the Task object
var Task = WinJS.Class.define(function () {
    var that = {};
    that.description = "This is my new task";
    that.dueDate = TodoList.firstOfNextMonth();
    that.priority = TodoList.Priority.Normal;
    that.status = "Not Started";
    that.percCompleted = 0;
    that.minPriority = TodoList.Priority.VeryLow;
    that.maxPriority = TodoList.Priority.VeryHigh;

    if (TodoList.settings.defaultPriority != "undefined")
        that.priority = TodoList.settings.defaultPriority;
    return that;
});



// App initializer
TodoList.init = function () {
    // Register handler for resize events
    window.onresize = addEventListener('resize', TodoList.onResize, false);

    // Register handler for buttons
    document.getElementById("buttonCancel").addEventListener("click", TodoList.cancelTaskClick);
    document.getElementById("buttonDeleteTask").addEventListener("click", TodoList.deleteTaskClick);
    document.getElementById("buttonNewTask").addEventListener("click", TodoList.newTaskClick);
    document.getElementById("buttonAddTask").addEventListener("click", TodoList.addTaskClick);
    document.getElementById("buttonShare").addEventListener("click", TodoList.shareClick);

    // Initialization of Share source contract
    var view = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
    view.addEventListener("datarequested", function (e) {
        var currentTask = TodoList.getTaskFromUI();

        if (currentTask.description.length === 0) {
            e.request.failWithDisplayText("You need to indicate a description of the task.");
            return;
        }
        TodoList.shareDataAsHtml(e, currentTask);
        TodoList.shareDataAsPlainText(e, currentTask);
    });

    // Load settings and initialize view
    TodoList.settings.load();
    TodoList.setupTaskList();
    TodoList.populateTaskList();
}


TodoList.setupTaskList = function () {
    var listview = document.getElementById("task-listview").winControl;
    listview.itemTemplate = document.getElementById("task-listitem");
    listview.addEventListener("iteminvoked", TodoList.taskSelected);
}

TodoList.populateTaskList = function () {
    //var tasks = [
    //        { description: "Task #1" },
    //        { description: "Task #2" },
    //        { description: "Task #3" }
    //];

    var tasks = new Array(); 
    var localFolder = Windows.Storage.ApplicationData.current.roamingFolder;
    localFolder.getFilesAsync()
                .then(function (files) {
                    var io = Windows.Storage.FileIO;
                    files.forEach(function (file) {
                        io.readTextAsync(file)
                        .then(function (json) {
                            var task = TodoList.deserializeTask(json);
                            tasks.push(task);
                        })
                        .then(function () {
                            var bindingList = new WinJS.Binding.List(tasks);
                            bindingList = bindingList.createSorted(function (lhs, rhs) {
                                return lhs.dueDate > rhs.dueDate;
                            });
                            var listview = document.getElementById("task-listview").winControl;
                            listview.itemDataSource = bindingList.dataSource;
                        });
                    });
                })
}

TodoList.shareDataAsPlainText = function (e, task) {
    var request = e.request;
    request.data.properties.title = "TO DO";
    request.data.properties.description = task.description;
    request.data.setText(task.description + " due by " + task.dueDate.toLocaleDateString());
}

TodoList.shareDataAsHtml = function (e, task) {
    var request = e.request;
    request.data.properties.title = "TO DO";
    request.data.properties.description = task.description;

    // Load the HTML layout (and run data binding)
    var elem = document.getElementById("shareHtml");
    WinJS.Binding.processAll(elem, task);
    var rawHtml = elem.innerHTML;

    // Make the raw HTML compliant with Windows 8 requirements
    var html = Windows.ApplicationModel.DataTransfer.HtmlFormatHelper.createHtmlFormat(rawHtml);
    request.data.setHtmlFormat(html);

    // This extra work is necessary ONLY if the HTML references an image.

    // This is the URL of the image as in the HTML block. You transform it in a URI object
    // and store it as an in-memory stream in the DataPackage resources.
    var localImage = "ms-appx:///images/todolist-icon.png";
    var url = new Windows.Foundation.Uri(localImage);
    var streamRef = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(url);
    request.data.resourceMap[localImage] = streamRef;
}

TodoList.addTaskClick = function () {
    var currentTask = TodoList.getTaskFromUI();
    if (!TodoList.validateInput(currentTask)) {
        TodoList.alert("There is something wrong with the data you entered.");
        return;
    }

    TodoList.displaySummary(currentTask);
}

TodoList.newTaskClick = function () {
    TodoList.displayTask(new Task());
}

TodoList.cancelTaskClick = function () {
    var editor = document.getElementById("editor-container");
    editor.style.display = "none";
    var listview = document.getElementById("task-listview").winControl;
    listview.selection.clear();
}

TodoList.deleteTaskClick = function () {
    var message = "Are you sure you want to delete the task?";
    var msg = new Windows.UI.Popups.MessageDialog(message);
    msg.commands.append(new Windows.UI.Popups.UICommand("Yes, proceed!", TodoList.deleteTask));
    msg.commands.append(new Windows.UI.Popups.UICommand("No, I'm not sure...", function() {}));
    msg.defaultCommandIndex = 1;
    msg.showAsync();
}

TodoList.deleteTask = function () {
    var task = TodoList.getTaskFromUI();
    var name = task.description;

    var localFolder = Windows.Storage.ApplicationData.current.roamingFolder;
    localFolder.getFileAsync(name)
        .then(function (file) {
            file.deleteAsync().then(function () {
                TodoList.cancelTaskClick();
                TodoList.populateTaskList();
            });
        });
}

TodoList.shareClick = function () {
    Windows.ApplicationModel.DataTransfer.DataTransferManager.showShareUI();
};

TodoList.alert = function (message) {
    var alertDialog = new Windows.UI.Popups.MessageDialog(message);
    alertDialog.showAsync();
}

TodoList.displayTask = function (task) {
    TodoList.performInitialBinding(task);

    // Ensure the editor is visible
    var editor = document.getElementById("editor-container");
    editor.style.display = "block";
}

TodoList.taskSelected = function (eventInfo) {
    eventInfo.detail.itemPromise.then(function (item) {
        TodoList.displayTask(item.data);
    });
}

TodoList.performInitialBinding = function (task) {

    // Enable binding on the HTML element(s) of choice 
    var bindableElement = document.getElementById("task-editor");
    WinJS.Binding.processAll(bindableElement, task);

    var datePicker = document.getElementById("taskDueDate").winControl;
    datePicker.current = task.dueDate;
    var select = document.getElementById("taskStatus");
    select.selectedIndex = TodoList.getIndexFromStatus(task.status);
}

TodoList.firstOfNextMonth = function () {
    var d = new Date();
    d.setDate(d.getDate() + 31);
    var year = d.getFullYear();
    var month = d.getMonth();
    var day = 1;
    var newDate = new Date(year, month, day);
    return newDate;
}

TodoList.getIndexFromStatus = function (option) {
    switch (option) {
        case "In progress": return 1;
        case "Completed": return 2;
        default: return 0;
    }
}

TodoList.getTaskFromUI = function () {
    var task = new Task();

    var taskSubject = document.getElementById("taskSubject");
    task.description = taskSubject.value;

    var taskDueDate = document.getElementById("taskDueDate").winControl;
    task.dueDate = taskDueDate.current;

    var taskPriority = document.getElementById("taskPriority");
    task.priority = taskPriority.value;

    var taskStatus = document.getElementById("taskStatus");
    task.status = taskStatus.options[taskStatus.selectedIndex].value;

    var taskPercCompleted = document.getElementById("taskPercCompleted");
    task.percCompleted = taskPercCompleted.value;

    return task;
}

TodoList.validateInput = function (task) {
    // Check description is NOT empty
    if (task.description.length == 0)
        return false;

    // Check date is a future date
    if (task.dueDate <= new Date())
        return false;

    // Check if perc-completed is >0 and task not started or 100 done but not completed
    if ((task.percCompleted > 0 && task.status == "Not Started")  ||
        (task.percCompleted == 100 && task.status != "Completed") ||
        (task.percCompleted != 100 && task.status == "Completed"))
        return false;

    return true;
}

TodoList.displaySummary = function (task) {
    // Prepare the content for the flyout
    var bindableElement = document.getElementById("flyoutSummary");
    WinJS.Binding.processAll(bindableElement, task);

    // Display the flyout
    var anchor = document.getElementById("buttonAddTask");
    var flyoutSummary = document.getElementById("flyoutSummary").winControl;
    flyoutSummary.show(anchor);
}

TodoList.onResize = function (e) {
    var snappedLayout = document.getElementById("list-of-tasks");
    var defaultLayout = document.getElementById("task-editor");
    var viewState = Windows.UI.ViewManagement.ApplicationViewState.value;
}

TodoList.pickFileAndSaveTask = function () {
    var task = TodoList.getTaskFromUI();

    var localFolder = Windows.Storage.ApplicationData.current.roamingFolder;
    var name = task.description;
    var io = Windows.Storage.FileIO;
    var fileOptions = Windows.Storage.CreationCollisionOption;
    localFolder.createFileAsync(name, fileOptions.replaceExisting)
                .done(function (file) {
                            io.writeTextAsync(file, TodoList.serializeTask(task))
                            .done(function () {
                                TodoList.alert("All done!");
                                TodoList.populateTaskList();
                                TodoList.cancelTaskClick();
                            })
                        },
                        function (error) {
                            TodoList.alert("Unable to create file. Sorry about that!");
                        });
}

TodoList.serializeTask = function (task) {
    //return task.description + "|" +
    //    task.dueDate + "|" + 
    //    task.priority + "|" + 
    //    task.status + "|" + 
    //    task.percCompleted;
    return JSON.stringify(task);
}

TodoList.deserializeTask = function (json) {
    var task = JSON.parse(json);
    task.dueDate = new Date(task.dueDate);
    return task;
}

// Converters
TodoList.dateForDisplay = WinJS.Binding.converter(function (value) {
    return value.toLocaleDateString();
});
TodoList.priorityForDisplay = WinJS.Binding.converter(function (value) {
    switch (value) {
        case TodoList.Priority.VeryLow:
            return "Very low";
        case TodoList.Priority.VeryHigh:
            return "Very high";
        case TodoList.Priority.Low:
            return "Low";
        case TodoList.Priority.High:
            return "High";
        default:
            return "Normal";
    }
});
TodoList.percForDisplay = WinJS.Binding.converter(function (value) {
    return value + "%";
});